/* main.c - initialisation functions
 * AbsVolume was programmed by Paul Sherman, derived in near 
 * entirety from the program "volumecontrol" by Rob Eberts
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2.
 * source code is available at <http://www.pcbypaul.com>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include "main.h"
#include "actions.h"
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "callbacks.h"
#include <linux/soundcard.h>
//#include <sys/types.h>
//#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>

int mixer_fd;

static char * mixer_xpm[] = {
"20 16 8 1",
" 	c None",
".	c #000000",
"+	c #808000",
"@	c #808080",
"#	c #00ED2D",
"$	c #FFFF00",
"%	c #C8C8C8",
"&	c #FFFFFF",
"       ..           ",
"      +@.@   ##.##  ",
"     +$.%%   ##.##  ",
"    +$@@%&@  ##.##  ",
"   +$&.%&&@  ##.##  ",
" ++$&$.%&&@  .....  ",
"+&%&$&..&&@  .###.  ",
"+$%$&$.%.&@  .....  ",
"+$%&$&.@.&@  ##.##  ",
".+@$&$..&&@  ##.##  ",
" ..+$&.%&&@  ##.##  ",
"   .+$.%&&@  ##.##  ",
"    .+@@&&@  ##.##  ",
"     .+.%%@  ##.##  ",
"      .+.@   ##.##  ",
"       ..           "};


extern int mixer_fd;
StereoVolume tmpvol;
struct originalset orig;
GtkTooltips *tips;

GtkWidget *window1;
GtkWidget *vbox1;
GtkWidget *hbox1;
GtkWidget *spinbutton1;
GtkWidget *hbox3;
GtkWidget *vscale1;
GtkAccelGroup *accel_group;
GtkAdjustment *vol_adjustment;
GtkWidget *hbox4;
GdkPixbuf *icon0;
GdkPixbuf *pixbuf;
GtkWidget *image;
GtkWidget *hbox5;
GtkWidget *mixBtn;
GtkWidget *image1;
GError *error=NULL;
GtkLabel *label;
GtkWidget *file_selection_box;
gchar *filename;

#define CONFIG_FILE_NAME ".absolumerc"

gchar *find_rc_file(void) {
    gchar *rc_file = NULL;
  	if(getenv("HOME")) {
		rc_file = g_strdup_printf("%s/%s", getenv ("HOME"), CONFIG_FILE_NAME);
    	} else { /* unable to find $HOME, assuming current directory */
    		rc_file = g_strdup(CONFIG_FILE_NAME);
        }
        return rc_file;
}


void store_filename(filename)
{
  /*-- Extract filename from the file selection dialog --*/
 // filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection_box));
  gchar *rc_file;
  rc_file = find_rc_file();
  FILE *fp = fopen (rc_file, "w");
  if (fp == NULL) return;
  fprintf (fp, "%s\n", filename);
  fclose (fp);
  g_free(rc_file);
}

void on_mixer(void)
{
	gchar *rc_file;
	rc_file = find_rc_file();
	FILE *fp;
    fp = fopen(rc_file, "r");
    if (NULL != fp)
    {
        char app_path[256];
        fgets(app_path, sizeof(app_path)-1, fp);
        fclose(fp);
        char *new_line = strchr(app_path, '\n');
        if (new_line) *new_line = '\0';
		
	    if((access(app_path, X_OK)) == -1)
        {	
		 GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window1),
                                		  GTK_DIALOG_DESTROY_WITH_PARENT,
                                		  GTK_MESSAGE_ERROR,
                                		  GTK_BUTTONS_CLOSE,
                                		  "\nMixer App selected is not valid.\nPlease select another\n");
		 gtk_dialog_run (GTK_DIALOG (dialog));
		 gtk_widget_destroy (dialog);		
		return;
		}
		if (strcmp(app_path,"/usr/bin/alsamixer") == 0)
		{
			const char *cmd0 = "xterm -e /usr/bin/alsamixer&";
			//gtk_widget_hide (window1);
			system(cmd0);
			gtk_main_quit();	
		} else {
			//gtk_widget_hide (window1);
			strcat(app_path, "&");
			system(app_path);
			gtk_main_quit();
			
		}
    } else {
	
		int no_alsamixergui = system("which alsamixer-qt4 | grep /alsamixer-qt4");
		int no_alsamix = system("which alsamixer | grep /alsamixer");
		if (no_alsamixergui) {
 			if (no_alsamix) {
				 GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window1),
                                				  GTK_DIALOG_DESTROY_WITH_PARENT,
                                				  GTK_MESSAGE_ERROR,
                                				  GTK_BUTTONS_CLOSE,
                                				  "\nNo mixer application was not found on your system.\n\nYou will need to install either alsamixer-qt4 or alsamixer if you wish to use a mixer from the volume control.");
				 gtk_dialog_run (GTK_DIALOG (dialog));
				 gtk_widget_destroy (dialog);
			} else {

				  const char *cmd1 = "xterm -e /usr/bin/alsamixer&";
				  //gtk_widget_hide (window1);
				  system(cmd1);	
				  gtk_main_quit();
			}
    	} else {
			const char *cmd = "/usr/local/bin/alsamixer-qt4&";
			//gtk_widget_hide (window1);
			system(cmd);
			gtk_main_quit();
		}
	}
}

void create_file_selection(void) 
{
    // file_selection_box = gtk_file_selection_new("Select MIXER application");
    file_selection_box = gtk_file_chooser_dialog_new("Select MIXER application", NULL,
                    						 GTK_FILE_CHOOSER_ACTION_OPEN,
                    						 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                    						 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                    						 NULL);

	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_selection_box), "/usr/bin");
	if (gtk_dialog_run (GTK_DIALOG (file_selection_box)) == GTK_RESPONSE_ACCEPT) {
		char *fn;
		fn = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_selection_box));
		store_filename(fn);
		g_free (fn);
	}
	gtk_widget_destroy (file_selection_box);
}

static void button_pressed_callback(GtkWidget *widget, GdkEvent *event, gpointer img)
{
	if (event->button.button == 1) {
        on_mixer();
	} else { 
		create_file_selection();
	}
} 

GtkWidget*
create_window1 (void)
{
  accel_group = gtk_accel_group_new ();

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (window1, 32, 200);
  gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
													 
  gtk_window_set_resizable (GTK_WINDOW (window1), FALSE);
  gtk_window_set_position (GTK_WINDOW (window1),GTK_WIN_POS_MOUSE);
  gtk_window_set_decorated (GTK_WINDOW (window1), FALSE);

  pixbuf = gdk_pixbuf_new_from_xpm_data((const char**)mixer_xpm);
  image1 = gtk_image_new_from_pixbuf(pixbuf);
  tips = gtk_tooltips_new ();
  
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  
  vol_adjustment=GTK_ADJUSTMENT (gtk_adjustment_new (1, 0, 100, 1, 10, 0));
  /* get original adjustments */
  get_current_levels();

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (vol_adjustment),1,0); /* ajustment, climb rate, digits */
 
 // spinbutton is instantiated, but not shown ****
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton1, FALSE, FALSE, 0);
  gtk_widget_set_size_request (spinbutton1, 36, -1);
  g_signal_connect ((gpointer) spinbutton1, "value_changed",G_CALLBACK (on_spinbutton1_button_release_event),NULL);
  g_object_set_data_full(G_OBJECT (window1), "spinbutton1",gtk_widget_ref (spinbutton1),(GDestroyNotify) gtk_widget_unref);
	
  hbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox4, FALSE, FALSE, 0);
  
  mixBtn = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(mixBtn), image1);
  gtk_button_set_relief(GTK_BUTTON (mixBtn),GTK_RELIEF_NONE);
  gtk_widget_set_size_request(image1, 20, 20);
  gtk_widget_show(mixBtn);
  gtk_widget_show(image1);
  gtk_tooltips_set_tip (tips, mixBtn," MIXER\nClick to run mixer\nRight-click to change default mixer", NULL);
  gtk_box_pack_start (GTK_BOX (hbox4), mixBtn, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(mixBtn), "button_press_event", G_CALLBACK(button_pressed_callback), NULL);
  
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 0);

  vscale1 = gtk_vscale_new (GTK_ADJUSTMENT (vol_adjustment));
  gtk_widget_show (vscale1);
  gtk_range_set_inverted (GTK_RANGE (vscale1), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox3), vscale1, TRUE, TRUE, 0);
  gtk_scale_set_draw_value (GTK_SCALE (vscale1), FALSE);
  g_signal_connect ((gpointer) vscale1, "button_release_event",G_CALLBACK (on_vscale1_button_release_event),vol_adjustment);
  g_signal_connect(vscale1, "value_changed", G_CALLBACK(set_tip), NULL);
  g_object_set_data_full(G_OBJECT (window1), "vscale1",gtk_widget_ref (vscale1),(GDestroyNotify) gtk_widget_unref);
  gtk_tooltips_set_tip (tips, vscale1,"Level", NULL);
		    		   
  g_object_set_data (G_OBJECT (window1), "window1", window1);
  g_object_set_data_full (G_OBJECT (window1), "vbox1",gtk_widget_ref (vbox1), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox1",gtk_widget_ref (hbox1), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox3",gtk_widget_ref (hbox3), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox4",gtk_widget_ref (hbox4), (GDestroyNotify) gtk_widget_unref);
  
  gtk_window_add_accel_group (GTK_WINDOW (window1), accel_group);
  
  
  gtk_widget_set_events (window1, GDK_KEY_PRESS_MASK);
  g_signal_connect (GTK_WINDOW (window1), "key_press_event", G_CALLBACK (do_me), NULL);


//  g_signal_connect(window1, "key-press-event", G_CALLBACK(hide_me), NULL);
//  g_signal_connect(window1, "button_press_event", G_CALLBACK(hide_me), NULL); 
  gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window1), TRUE);
  set_tip();
  gtk_widget_grab_focus(vscale1);
  g_signal_connect(GTK_OBJECT(vscale1),"focus_out_event", GTK_SIGNAL_FUNC(FocusOutEvent), NULL);
  return window1;
}

void get_current_levels() {
  /*   ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &tmpvol); */
  ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &tmpvol);
  /* SOUND_MIXER_PCM or SOUND_MIXER_VOLUME*/

  orig.mainvol=tmpvol.left;
  gtk_adjustment_set_value(GTK_ADJUSTMENT(vol_adjustment), (double) tmpvol.left);

}
void FocusOutEvent() {
	gtk_main_quit();
}

void about() {
	GtkWidget *dialog2 = gtk_about_dialog_new();
	gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(dialog2), "AbsVolume");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog2), "6.0"); 
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog2), "(c)2004-2010 by Paul Sherman");
	GdkPixbuf *pixbuf_about = gdk_pixbuf_new_from_xpm_data((const char**)mixer_xpm);
	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(dialog2), pixbuf_about);
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog2), "Simple volume control to run from a taskbar,\nvolume (PCM) changes are session specific.");
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog2), "http://www.absolutelinux.org");
	g_object_unref(pixbuf_about), pixbuf = NULL;
	gtk_dialog_run (GTK_DIALOG (dialog2));
	gtk_widget_destroy (dialog2);
}

void help() {
	GtkWidget *dialog1 = gtk_message_dialog_new (GTK_WINDOW(window1), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "\nAbsvolume:\nsimple volume control.\n\nVolume changes are NOT saved to system,\nso they are temporary to current session/login.\n\nMove slider to adjust volume (PCM)\n\nThe button atop the slider (speaker icon)\nopens your mixer application\n\nRight-clicking speaker button allows you \nto select a custom mixer application.\n\nHitting SPACE bar will MUTE and close.\n\"h\" or \"F1\" displays this help message\n\"a\" dislays an about dialog\n");
	gtk_window_set_title(GTK_WINDOW(dialog1), "absvolume help");
	gtk_dialog_run (GTK_DIALOG (dialog1));
	gtk_widget_destroy (dialog1);
}


void do_me(GtkWidget *widget, GdkEventKey *ev) {
  switch (ev->keyval)
  {
  case GDK_a:
	about();
	break;
  case GDK_h:
	help();
	break;
  case GDK_F1:
	help();
	break;
  case GDK_space:
    quick_mute();
	gtk_main_quit();
  default:
    gtk_main_quit();
  }
}

void set_tip() {
	int num = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton1));
	char result[5];
	sprintf( result, "%d", num );
	gtk_tooltips_set_tip (tips, vscale1, result, NULL);
}

main (int argc, char *argv[])
{

  system("pidof -o $$ -o $PPID absvolume > /tmp/PID_TMP");
  system("kill `cat /tmp/PID_TMP` 2>/dev/null");
  system("rm -f /tmp/PID_TMP");

  mixer_fd = open ("/dev/mixer", O_RDWR, 0);
  if (mixer_fd < 0)
    printf ("Error opening mixer device\n"), exit (1);

  if (argc > 1) 
  {
    args(argc, argv);
    return 0;
  }

  GtkWidget *window1;
  gtk_init (&argc, &argv);
  window1 = create_window1();
  gtk_widget_show (window1);
  
  gtk_main ();
  close(mixer_fd);
  return 0;
}


